package net.knarcraft.minigames.arena.parkour;

import net.knarcraft.minigames.arena.StorageKey;
import org.jetbrains.annotations.NotNull;

/**
 * A representation of each key used for storing arena data
 */
public enum ParkourArenaStorageKey implements StorageKey {

    /**
     * The key for an arena's id
     */
    ID("arenaId"),

    /**
     * The key for an arena's name
     */
    NAME("arenaName"),

    /**
     * The key for an arena's spawn location
     */
    SPAWN_LOCATION("arenaSpawnLocation"),

    /**
     * The key for an arena's exit location
     */
    EXIT_LOCATION("arenaExitLocation"),

    /**
     * The key for the type of this arena's win block
     */
    WIN_BLOCK_TYPE("winBlockType"),

    /**
     * The key for this arena's win location (overrides win block type)
     */
    WIN_LOCATION("winLocation"),

    /**
     * The key for this arena's kill plane blocks (overrides the config)
     */
    KILL_PLANE_BLOCKS("killPlaneBlocks"),

    /**
     * The key for this arena's obstacle blocks (overrides the config)
     */
    OBSTACLE_BLOCKS("obstacleBlocks"),

    /**
     * The key for this arena's checkpoint locations
     */
    CHECKPOINTS("checkpoints"),

    /**
     * The hey for this arena's data
     */
    DATA("arenaData"),

    /**
     * The key for this arena's rewards
     */
    REWARDS("rewards"),

    /**
     * The key for this arena's maximum players
     */
    MAX_PLAYERS("maxPlayers"),

    /**
     * The key for this arena's allowed damage causes
     */
    ALLOWED_DAMAGE_CAUSES("allowedDamageCauses"),

    /**
     * The key for this arena's loss trigger damage causes
     */
    LOSS_TRIGGER_DAMAGE_CAUSES("lossTriggerDamageCauses"),
    ;

    private final @NotNull String key;

    /**
     * Instantiates a new arena storage key
     *
     * @param key <p>The string path of the configuration key this value represents.</p>
     */
    ParkourArenaStorageKey(@NotNull String key) {
        this.key = key;
    }

    @Override
    public @NotNull String getKey() {
        return this.key;
    }

}
